package com.bull.cimero.pluginEditor.editors.commands;

import java.util.List;
import org.eclipse.gef.commands.Command;
import com.bull.cimero.pluginEditor.editors.model.Connection;
import com.bull.cimero.pluginEditor.editors.model.GeneriqueCimeroModel;

/**
 * This class is used to create the command.
 * to create connection beetween two objects.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class ConnectionCreateCommand extends Command {

	/** current connection. */
	private Connection connection;
	/** source of the current connection. */
	private GeneriqueCimeroModel source;
	/** target of the current connection. */
	private GeneriqueCimeroModel target;
	/**The type of connection.*/
	private boolean connectionBottom = false;

   /**
    * set the source parameter.
    * @param sourceParam the source given in parameter
    */
   public final void setSource(final GeneriqueCimeroModel sourceParam) {
      this.source = sourceParam;
   }

   /**
    * set the connection parameter.
    * @param connectionParam the connection given in parameter
    */
   public final void setConnection(final Connection connectionParam) {
      this.connection = connectionParam;
   }

   /**
    * set the target parameter.
    * @param targetParam the target given in parameter
    */
   public final void setTarget(final GeneriqueCimeroModel targetParam) {
      this.target = targetParam;
   }

   /**
    * get the label of the function.
    * @return String : the label
    */
   public final String getLabel() {
      return "Create Connection";
   }

   /**
    * function to verify the method is executable.
    * @return boolean : the executability
    */
   public final boolean canExecute() {
      if (source.equals(target)) {
         return false;
      }
      // Check for existence of connection already
      List connections = this.source.getOutgoingConnections();
      for (int i = 0; i < connections.size(); i++) {
         if (((Connection) connections.get(i)).getTarget().equals(target)) {
            return false;
         }
      }
      return true;
   }

   /**
    * execute the method.
    */
   public final void execute() {
      this.connection = new Connection(source, target, connectionBottom);
   }

   /**
    * undo the method.
    */
   public final void undo() {
      this.source.removeOutput(this.connection);
      this.target.removeInput(this.connection);
   }

   /**
    * redo the method.
    */
   public final void redo() {
      this.source.addOutput(this.connection);
      this.target.addInput(this.connection);
   }

   /**
    * get the connection.
    * @return Connection : the connection
    */
   public final Connection getConnection() {
		return connection;
   }

   /**
    * get the source.
    * @return GeneriqueCimeroModel : the source
    */
   public final GeneriqueCimeroModel getSource() {
		return source;
   }

   /**
    * get the target.
    * @return GeneriqueCimeroModel : the target
    */
   public final GeneriqueCimeroModel getTarget() {
		return target;
   }

	/**
	 * @param connectionBottomP the connectionBottom to set
	 */
	public final void setConnectionBottom(final boolean connectionBottomP) {
		this.connectionBottom = connectionBottomP;
	}
}
