package com.bull.cimero.pluginEditor.editors.model.BCModel;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
/**
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */

public class TCPBC extends BindingComponentModel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private static final String BC_NAME = "TCPBC".intern();
	private TCPModelHelper helper;
	private static final String ID = "TCPBCModel".intern();
	private boolean createdByPipeline = false;

	public TCPBC() {
		super(ID);
		setProperty(NAME, BC_NAME);
		setProperty(INOUT, 0);
		if (setInput(false)) {
			setOutput(true);
		}
		getHelper().setDefaultProperties(this);
	}
	
	
	private TCPModelHelper getHelper() {
		if (helper == null) {
			helper = new TCPModelHelper();
			helper.setInput(false);
			helper.setOutput(true);
			helper.setBindingComponent(true);
		}
		return helper;
	}
	
	@Override
	protected void createDescriptor() {
		IPropertyDescriptor pDescr;
		pDescr = new TextPropertyDescriptor(NAME, " Component Name");
		addDescriptor(pDescr);
		
		getHelper().setDescriptors(this);
	}

	@Override
	public Object getPropertyValue(Object id) {
		if (TCPModelHelper.BASE64_ENCODE.equals(id)) {
			Object value = getProperty(TCPModelHelper.BASE64_ENCODE);
			if (value == null || !(value instanceof Integer)) {
				setProperty(TCPModelHelper.BASE64_ENCODE, 0);
				return 0;
			}
		}
		if (TCPModelHelper.BASE64_DECODE.equals(id)) {
			Object value = getProperty(TCPModelHelper.BASE64_DECODE);
			if (value == null || !(value instanceof Integer)) {
				setProperty(TCPModelHelper.BASE64_DECODE, 0);
				return 0;
			}
		}
		if (TCPModelHelper.OUT_NM_ENVELOPE.equals(id)) {
			return getOutNmEnvelope();
		}
		if (TCPModelHelper.IN_NM_ENVELOPE.equals(id)) {
			return getInNmEnvelope();
		}
		if (TCPModelHelper.LOG_CONNECTIONS.equals(id)) {
			Object value = getProperty(TCPModelHelper.LOG_CONNECTIONS);
			return value != null? value : 1;
		}
		if (TCPModelHelper.LOG_DATA.equals(id)) {
			Object value = getProperty(TCPModelHelper.LOG_DATA);
			return value != null? value : 1;
		}
		if (TCPModelHelper.LOG_DATA_AS_HEX.equals(id)) {
			Object value = getProperty(TCPModelHelper.LOG_DATA_AS_HEX);
			return value != null? value : 1;
		}
		if (TCPModelHelper.EXTRA_INFORMATION.equals(id)) {
			Object value = getProperty(TCPModelHelper.EXTRA_INFORMATION);
			return value != null? value : 1;
		}
		return super.getPropertyValue(id);
	}
	
	@Override
	public void setPropertyValue(Object id, Object value) {
		if (INOUT == id) {
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		}
		super.setPropertyValue(id, value);
	}
	
	@Override
	protected Object clone() throws CloneNotSupportedException {
		return super.clone();
	}

	@Override
	public String getComponentId() {
		return ID;
	}

	@Override
	public boolean isValide(boolean showLog) {
		return getHelper().isValide(this, showLog);
	}

	public void resetPropertyValue(Object id) {
	}
	
	/*  */
	public String getLocalPort() {
		return (String)getProperty(TCPModelHelper.LOCAL_PORT);
	}
	
	public String getLocalAddress() {
		return (String)getProperty(TCPModelHelper.LOCAL_ADDRESS);
	}
	
	public String getRemotePort() {
		return (String)getProperty(TCPModelHelper.REMOTE_PORT);
	}
	
	public String getRemoteHost() {
		return (String)getProperty(TCPModelHelper.REMOTE_HOST);
	}
	
	public String getIncomingWrapper() {
		int inWrapper = ((Integer)getProperty(TCPModelHelper.INCOMING_WRAPPER)).intValue();
		return TCPModelHelper.WRAPPER_COMBO_VALUES[inWrapper];
	}
	
	public String getIncomingHeader() {
		return (String)getProperty(TCPModelHelper.INCOMING_HEADER);
	}

	public String getIncomingTrailer() {
		return (String)getProperty(TCPModelHelper.INCOMING_TRAILER);
	}

	public String getStripWrapping() {
		int intStripWrapping = ((Integer)getProperty(TCPModelHelper.STRIP_WRAPPING)).intValue();
		return TCPModelHelper.BOOLEAN_COMBO_VALUES[intStripWrapping];
	}
	
	public String getConnNumber() {
		return (String)getProperty(TCPModelHelper.CONN_NUMBER);
	}
	
	public String getUseSSL() {
		int intUseSSL = ((Integer)getProperty(TCPModelHelper.USE_SSL)).intValue();
		return TCPModelHelper.BOOLEAN_COMBO_VALUES[intUseSSL];
	}
	
	public String getUseSSLClientMode() {
		int intUseSSLClientMode = ((Integer)getProperty(TCPModelHelper.USE_SSL_CLIENT_MODE)).intValue();
		return TCPModelHelper.BOOLEAN_COMBO_VALUES[intUseSSLClientMode];
	}
	
	public String getOutgoingWrapper() {
		int intOutWrapper = ((Integer)getProperty(TCPModelHelper.OUTGOING_WRAPPER)).intValue();
		return TCPModelHelper.WRAPPER_COMBO_VALUES[intOutWrapper];
	}
	
	public String getOutgoingHeader() {
		return (String)getProperty(TCPModelHelper.OUTGOING_HEADER);
	}

	public String getOutgoingTrailer() {
		return (String)getProperty(TCPModelHelper.OUTGOING_TRAILER);
	}

	public String getPointName() {
		return (String)getProperty(TCPModelHelper.POINT_NAME);
	}

	public String getPointType() {
		return isInput()? TCPModelHelper.POINT_TYPE_SERVER: TCPModelHelper.POINT_TYPE_CLIENT;
	}

	public String getPointMode() {
		int intPointMode = ((Integer)getProperty(TCPModelHelper.POINT_MODE)).intValue();
		return TCPModelHelper.OPERATION_MODE_COMBO_VALUES[intPointMode];
	}

	public String getResponseTimeout() {
		return (String)getProperty(TCPModelHelper.RESPONSE_TIMEOUT);
	}	

	public String getRetryCount() {
		return (String)getProperty(TCPModelHelper.RETRY_COUNT);
	}	

	public String getFailAction() {
		int intFailAction = ((Integer)getProperty(TCPModelHelper.FAIL_ACTION)).intValue();
		return TCPModelHelper.FAIL_ACTION_COMBO_VALUES[intFailAction];
	}

	public String getRetryNumber() {
		return (String)getProperty(TCPModelHelper.RETRY_NUMBER);
	}	

	public String getRetryType() {
		int intRetryType = ((Integer)getProperty(TCPModelHelper.RETRY_TYPE)).intValue();
		return TCPModelHelper.RETRY_TYPE_COMBO_VALUES[intRetryType];
	}	

	public String getRetryDelay() {
		return (String)getProperty(TCPModelHelper.RETRY_DELAY);
	}	

	public String getOutNmEnvelope() {
		Object value = getProperty(TCPModelHelper.OUT_NM_ENVELOPE);
		return value != null?(String)value:"";
	}	

	public String getInNmEnvelope() {
		Object value = getProperty(TCPModelHelper.IN_NM_ENVELOPE);
		return value != null?(String)value:"";
	}
	
	public String getTcpOutInReceiverCalssName() {
		return (String)getProperty(TCPModelHelper.TCP_OUT_IN_RECEIVER_CLASSNAME);
	}
	public String getDefaultMep() {
		int intDefaultMep = ((Integer)getProperty(TCPModelHelper.DEFAULT_MEP)).intValue();
		return TCPModelHelper.MEP_COMBO_VALUES[intDefaultMep];
	}
	
	public String getIncomingEndianness() {
		int intIncomingEndianness = ((Integer)getProperty(TCPModelHelper.INCOMING_ENDIANNESS)).intValue();
		return TCPModelHelper.ENDIANNESS_COMBO_VALUES[intIncomingEndianness];
	}
	
	public String getOutgoingEndianness() {
		int intOutgoingEndianness = ((Integer)getProperty(TCPModelHelper.OUTGOING_ENDIANNESS)).intValue();
		return TCPModelHelper.ENDIANNESS_COMBO_VALUES[intOutgoingEndianness];
	}
	public String getConnectionMode() {
		try {
			int intConnectionMode = ((Integer)getProperty(TCPModelHelper.CONNECTION_MODE)).intValue();
			return TCPModelHelper.COMBO_CONNECTION_MODES[intConnectionMode];
		} catch (NullPointerException npe) {
			// the property was not previously setted
			setProperty(TCPModelHelper.CONNECTION_MODE, 0);
			return TCPModelHelper.COMBO_CONNECTION_MODES[0];
		}
	}
	public String getKeystoreFileName() {
		return (String)getProperty(TCPModelHelper.KEY_STORE_FILE_NAME);
	}
	
	public String getKeystorePassword() {
		return (String)getProperty(TCPModelHelper.KEY_STORE_PASSWORD);
	}
	
	public String getTruststoreFileName() {
		return (String)getProperty(TCPModelHelper.TRUST_STORE_FILE_NAME);
	}
	
	public String getTruststorePassword() {
		return (String)getProperty(TCPModelHelper.TRUST_STORE_PASSWORD);
	}
	
	public String getBase64Encode() {
		return getBooleanValue(TCPModelHelper.BASE64_ENCODE, 0);
	}
	
	public String getBase64Decode() {
		return getBooleanValue(TCPModelHelper.BASE64_DECODE, 0);
	}
	
	public String getLogData() {
		return getBooleanValue(TCPModelHelper.LOG_DATA, 1);
	}
	
	public String getLogConnection() {
		return getBooleanValue(TCPModelHelper.LOG_CONNECTIONS, 1);
	}
	
	public String getLogDataAsHex() {
		return getBooleanValue(TCPModelHelper.LOG_DATA_AS_HEX, 1);
	}
	
	public String getExtraInformation() {
		return getBooleanValue(TCPModelHelper.EXTRA_INFORMATION, 1);
	}
	
	private String getBooleanValue(String propertyName, int defaultIdx) {
		try {
			int intBoolean = ((Integer)getProperty(propertyName)).intValue();
			return TCPModelHelper.BOOLEAN_COMBO_VALUES[intBoolean];
		} catch (NullPointerException npe) {
			// the property was not previously setted
			setProperty(propertyName, defaultIdx);
			return TCPModelHelper.BOOLEAN_COMBO_VALUES[defaultIdx];
		}
	}

	public String getConnectionLogFileName() {
		Object value = getProperty(TCPModelHelper.CONNECTION_LOG_FILE_NAME);
		return value != null?(String)value:"";
	}


	public boolean isCreatedByPipeline() {
		return createdByPipeline;
	}


	public void setCreatedByPipeline(boolean createdByPipeline) {
		this.createdByPipeline = createdByPipeline;
	}
}
