package com.bull.cimero.pluginEditor.editors.model;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;

/**
 * class representing the diagram element.
 * (diagram model)
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class Diagram extends Element {

	/** serialization version. */
	static final long serialVersionUID = 1;
	/** properties. */
	public static final String NODES = "nodes";
	/** actual fields. */
	private List < GeneriqueCimeroModel > lstCimeroObject =
		new ArrayList < GeneriqueCimeroModel >();
	/**The name of this diagram.*/
	private String cimeroDiagramName = "";
	
	private String currentSelectedNode = null;

	/**
	 * default constructor.
	 */
	public Diagram() {
		super();
	}

	/**
	 * @param node the element to add
	 * @throws Exception if pb
	 */
	public final void addNode(final GeneriqueCimeroModel node)
		throws Exception {
		boolean add = lstCimeroObject.add(node);
		if (!add) {
			throw new Exception(
					"erreur lors de l'ajout de l'element node dans le diagram");
		}
		fireStructureChange(NODES, lstCimeroObject);
	}

	/**
	 * This function is used to delete an Object.
	 * @param node : the node to remove
	 */
	public final void removeNode(final GeneriqueCimeroModel node) {
		//delete the connection with this element
		if (node.isEntryConnection()) {
			node.removeAllInput();
		}
		if (node.isOutputConnection()) {
			node.removeAllOutput();
		}
		//delete the element
		lstCimeroObject.remove(node);
		fireStructureChange(NODES, lstCimeroObject);
	}

	/**
	 * get the list of cimero objects.
	 * @return List : the list
	 */
	public final List < GeneriqueCimeroModel > getLstCimeroObject() {
		return lstCimeroObject;
	}

	/**
	 * get the model you wanted.
	 * @param name : the name of the wanted model
	 * @return GeneriqueCimeroModel : the model wanted
	 */
	public final GeneriqueCimeroModel getGeneriqueModel(final String name) {
		for (int i = 0; i < lstCimeroObject.size(); i++) {
			GeneriqueCimeroModel tmp = ((GeneriqueCimeroModel)
								lstCimeroObject.get(i));
			if (name.compareTo(tmp.getName()) == 0) {
				return tmp;
			}
		}
		return null;
	}

	/**
	 * @return GeneriqueCimeroModel
	 * @throws IOException : if pb
	 */
	public final InputStream getAsStream() throws IOException {
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		ObjectOutputStream out = new ObjectOutputStream(os);
		out.writeObject(this);
		out.close();
		InputStream istream = new ByteArrayInputStream(os.toByteArray());
		os.close();
		return istream;
	}

	/**
	 * @param istream : the stream
	 * @return Diagram : the diagram created
	 * @throws IOException : if pb
	 * @throws ClassNotFoundException : if pb
	 */
	public static Diagram makeFromStream(final InputStream istream)
			throws IOException, ClassNotFoundException {
		ObjectInputStream ois = new ObjectInputStream(istream);
		Diagram diagram = (Diagram) ois.readObject();
		ois.close();
		return diagram;
	}

	/**
	 * get the name of the cimero diagram.
	 * @return the cimeroDiagramName
	 */
	public final String getCimeroDiagramName() {
		return cimeroDiagramName;
	}

	/**
	 * set the name of the cimero diagram.
	 * @param cimeroDiagramNameParam the name to set
	 */
	public final void setCimeroDiagramName(
					final String cimeroDiagramNameParam) {
		this.cimeroDiagramName = cimeroDiagramNameParam;
	}

	public String getCurrentSelectedNode() {
		return currentSelectedNode;
	}

	public void setCurrentSelectedNode(String currentSelectedNode) {
		this.currentSelectedNode = currentSelectedNode;
	}
	
	
	
}
