package com.bull.cimero.pluginEditor.editors.model.SEModel;

import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The MessageFilter Class.
 * This class is the representation of the message filter (EIP composant)
 * in Eclipse.
 * @author Jerome Camilleri
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class MessageFilter extends ServiceEngineModel {
    /** Serial UID. */
    private static final long serialVersionUID = 3L;
    /** Default name. */
    private static final String DEFAULTNAME = "myMessageFilter";
    /**The xpath rule.*/
    private static final String XPATHPREDICATE = "xpath rule";
    /**The name of the destination service property.*/
    private static final String NAMEDEST = "destination service";
    /**the id of this element.*/
    private static final String ID = "MESSAGEFILTER";

    /**
     * default constructor.
     */
    public MessageFilter() {
        super(DEFAULTNAME, 1, DEFAULTOUTIN);
        //set the properties
        setProperty(NAME, DEFAULTNAME);
        setProperty(XPATHPREDICATE, "");
        setProperty(NAMEDEST, "");
        setStandard(true);
    }

    /**
     * this function initialize the desciptor.
     */
    protected final void createDescriptor() {
        int indice = 1;
        addTextDescriptor(NAME, indice++ + " - name");
        addTextDescriptor(XPATHPREDICATE, indice++
                + " - xpath dedicate");
        String [] lstStr = getLstObjectDrawWithInputConnection();
        addComboBoxDescriptor(NAMEDEST, indice++
                          + " - select the destination service", lstStr);
    }

    /**
     * get the component id.
     * @return the component id.
     */
    public final String getComponentId() {
        return ID;
    }

    /**
     * test if this component is valide.
     * @param showLog : true if the log must be show.
     * @return true or false.
     */
    public final boolean isValide(final boolean showLog) {
        String msg = "";
        boolean returnResult = true;
        //name verification
        if (getName().compareTo("") == 0) {
              msg += "\n1 - Name => the messageFilter name cannot be null";
              returnResult = false;
        }
        //location URI verification
        if (getXpathRule().compareTo("") == 0) {
              msg += "\n2 - XPath predicate cannot be null "
            	  + "=> set the xpath predicate";
              returnResult = false;
        } else {
			//we test if XPath expression is valid
			if (!Ressources.testXPath(getXpathRule())) {
				msg += "\n2 - XPath predicate => expression not valid.";
	            returnResult = false;
			}
		}
        //destination verification
        if (getNameFirstDestination().compareTo("") == 0) {
              msg += "\n4 - Destination service => the destination of this "
                                + "element cannot be null!";
              returnResult = false;
        }
        if (!returnResult) {
              msg = msg.substring(1);
              msg = "ELEMENT NAME : " + getName() + "\n" + msg;
             if (showLog) {
            	 SpagicLogger.warning(msg);
             }
        }
        return returnResult;
    }

    /**
     * IPropertySource abstract method.
     * @return Object
     * @param id : the id of the property
     */
    public final Object getPropertyValue(final Object id) {
        if (NAMEDEST.equals(id)) {
            //we retreive the name of the destination object
            String destinationName = getNameFirstDestination();
            //return the num of the name  of the object
            String [] lstStr = getLstObjectDrawWithInputConnection();
            for (int i = 0; i < lstStr.length; i++) {
                if (lstStr[i].compareTo(destinationName) == 0) {
                    return i;
                }
            }
            return 0;
        } else {
            return super.getPropertyValue(id);
        }
    }

    /**
     * getXpathRule public method.
     * @return the component xpath rule.
     */
    public final String getXpathRule() {
        return (String) getProperty(XPATHPREDICATE);
    }

    /**
     * IPropertySource abstract method.
     * @param value : the value of the property
     * @param id : the id of the property
     */
    public final void setPropertyValue(final Object id, final Object value) {
        if (id == NAMEDEST) {
            //we retreive the destination name
            int selecNum = Integer.parseInt(value.toString());
            //retreive the list af all the objects name
            String [] lstStr = getLstObjectDrawWithInputConnection();
            //call the function to create a connection beetween
            //this object and the selected object
            if (!refreshDestinationConnection(lstStr[selecNum])) {
            	SpagicLogger.error("problem when created connection",
                        new Exception());
            }
        }
        super.setPropertyValue(id, value);
    }

	/**
	 * @see org.eclipse.ui.views.properties.IPropertySource#
	 * resetPropertyValue(java.lang.Object)
	 * @param id : the id of this object.
	 */
	public void resetPropertyValue(final Object id) {
		//
	}
}
