package com.bull.cimero.pluginEditor.utils.propertyCombo;

import org.eclipse.jface.viewers.LabelProvider;

/**
 * The label provider class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public class ComboBoxLabelproviderEditable extends LabelProvider {
	/**
	 * The array of String labels.
	 */
	private String[] values;
	/**The parent of this label.*/
	private ComboBoxPropertyEditable monCombo;

	/**
	 * @param valuesP the possible label values that this
	 * <code>ILabelProvider</code> may return.
	 * @param editor : the parent.
	 */
	public ComboBoxLabelproviderEditable(final String[] valuesP,
			final ComboBoxPropertyEditable editor) {
		this.values = valuesP;
		this.monCombo = editor;
	}

	/**
	 * @return the possible label values that this
	 * <code>ILabelProvider</code> may return.
	 */
	public final String[] getValues() {
		return values;
	}

	/**
	 * @param valuesP the possible label values that this
	 * <code>ILabelProvider</code> may return.
	 */
	public final void setValues(final String[] valuesP) {
		this.values = valuesP;
	}

	/**
	 * Returns the <code>String</code> that maps to the given
	 * <code>Integer</code> offset in the values array.
	 *
	 * @param element an <code>Integer</code> object whose value is a valid
	 * location within the values array of the receiver
	 * @return a <code>String</code> from the provided values array, or the
	 * empty <code>String</code>
	 * @see org.eclipse.jface.viewers.ILabelProvider#getText(java.lang.Object)
	 */
	public final String getText(final Object element) {
		if (monCombo != null) {
			String [] parentLabels = monCombo.getLabels();
			values = parentLabels;
		}
		if (element == null) {
			return ""; //$NON-NLS-1$
		}

		if (element instanceof Integer) {
			int index = ((Integer) element).intValue();
			if (index >= 0 && index < values.length) {
				return values[index];
			} else {
				return "et merdes"; //$NON-NLS-1$
			}
		}

		return ""; //$NON-NLS-1$
	}
}
