package com.bull.cimero.pluginEditor.utils.ressources;

import java.io.File;

/**
 * class used to manipulate file.
 * copy, remove, ...
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello ( andrea.zoppello@eng.it )
 */
public final class FileUtils {

	/** current FileUtils instance. */
	private static FileUtils fileUtils = null;
	/** width of bytes[]. */
	private static final int TABBYTESWIDTH = 512;
	/** height of bytes[]. */
	private static final int TABBYTESHEIGHT = 1024;

	/**
	 * default constructor.
	 */
	private FileUtils() {
		//nothing to do
	}

	/**
	 * method used to get the current instance of the class.
	 * @return FileUtils : the instance
	 */
	public static FileUtils getDefault() {
		if (fileUtils == null) {
			fileUtils = new FileUtils();
		}
		return fileUtils;
	}

	/**
	 * method used to copy a source file into a destination file.
	 * return true if no error.
	 * @param source : the source
	 * @param destination : the new file
	 * @return boolean : true if no error
	 */
	public boolean copyFile(
			final File source, final File destination) {
	    boolean resultat = false;
	    java.io.FileInputStream sourceFile = null;
	    java.io.FileOutputStream destinationFile = null;
	    try {
	        //creating the destination file
	        destination.createNewFile();
	        //open streams
	        sourceFile = new java.io.FileInputStream(source);
	        destinationFile = new java.io.FileOutputStream(destination);
	        //lecture
	        byte[] buffer = new byte [TABBYTESWIDTH * TABBYTESHEIGHT];
	        int nbLecture;
	        while ((nbLecture = sourceFile.read(buffer)) != -1) {
	             destinationFile.write(buffer, 0, nbLecture);
	        }
	        //no error
	        resultat = true;
	    } catch (java.io.FileNotFoundException f) {
	    	return false;
	    } catch (java.io.IOException e) {
	    	return false;
	    } finally {
	        //finally close the stream
	        try {
	             sourceFile.close();
	        } catch (Exception e) {
	        	return false;
	        }
	        try {
	             destinationFile.close();
	        } catch (Exception e) {
	        	return false;
		    }
	    }
	    return (resultat);
	}

	/**
	 * method used to copy a source file (path) into a destination file (path).
	 * return true if no error.
	 * @param sourcePath : the source path
	 * @param destinationPath : the new file (path)
	 * @return boolean : true if no error
	 */
	public boolean copyFile(
			final String sourcePath, final String destinationPath) {
		return copyFile(new File(sourcePath), new File(destinationPath));
	}

	/**
	 * method used to remove a file (path).
	 * return true if no error.
	 * @param filePath : the path to the file
	 * @return boolean : true if no error
	 */
	public boolean removeFile(final String filePath) {
		return removeFile(new File(filePath));
	}

	/**
	 * method used to remove a file.
	 * return true if no error.
	 * @param filePath : the file
	 * @return boolean : true if no error
	 */
	public boolean removeFile(final File filePath) {
		//if the file exists
		if (filePath.exists()) {
			//if it's really a file
			if (filePath.isFile()) {
				return filePath.delete();
			}
		}
		return false;
	}

	/**
	 * method used to test if a file exists.
	 * return true if it exists.
	 * @param filePath : the file
	 * @return boolean : true if exists
	 */
	public boolean testExistFile(final File filePath) {
		return filePath.exists();
	}

	/**
	 * method used to test if a file (path) exists.
	 * return true if it exists.
	 * @param filePath : the file
	 * @return boolean : true if exists
	 */
	public boolean testExistFile(final String filePath) {
		return testExistFile(new File(filePath));
	}

	/**
	 * return true is the filepath presents a file.
	 * @param filePath : the filepath
	 * @return true if the file exists
	 */
	public boolean isFile(final String filePath) {
		File myFile = null;
		try {
			myFile = new File(filePath);
			if (myFile.isFile()) {
				return true;
			}
		} catch (Exception e) {
			return false;
		}
		return false;
	}

	/**
	 * load a file.
	 * @param path : the path of the file.
	 * @return the inputstream.
	 */
	public static File loadFile(final String path) {
		File myFile = null;
		myFile = new File(path);
		return myFile;
	}
}
