/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import java.util.ArrayList;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.bean.ServiceRegistryConfiguration;
import org.spagic.utils.SpagicLogger;

public class ListServiceRegistryDialog extends SpagicTitleAreaDialog {
		
    private SashForm internalSash  = null; 
    
    private Table tableServiceRegistry = null;
    private TableViewer tbvServiceRegistry = null;
    private List<ServiceRegistryConfiguration> listServiceRegistry = null;
    private ServiceRegistryConfiguration selectedServiceRegistry = null;
    
	public ListServiceRegistryDialog(Shell parentShell) {
		super(parentShell);
		this.listServiceRegistry = new ArrayList<ServiceRegistryConfiguration>();
	}		

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Service Registry");
	    setMessage("Configure Service Registry");
	   
	  }

	 
	  

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
        this.tableServiceRegistry = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableServiceRegistry.setLinesVisible(true);
        this.tableServiceRegistry.setHeaderVisible(true);
        TableColumn column = null;
		column = new TableColumn(tableServiceRegistry, SWT.LEFT);
		column.setText("Name");
		column.setWidth(200);
		column = new TableColumn(tableServiceRegistry, SWT.LEFT);
		column.setText("Inquiry Url");
		column.setWidth(300);
		column = new TableColumn(tableServiceRegistry, SWT.LEFT);
		column.setText("Publish Url");
		column.setWidth(300);
		
        this.tbvServiceRegistry = new TableViewer(tableServiceRegistry);
        tbvServiceRegistry.setContentProvider(new ServiceRegistryContentProvider());
        tbvServiceRegistry.setLabelProvider(new ServiceRegistryLabelProvider());
        listServiceRegistry = getServiceRegistryList();
        tbvServiceRegistry.setInput(listServiceRegistry);    
        
        final ListServiceRegistryDialog  theDialog = this;
        
        this.tableServiceRegistry.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        	};
        	
        });
        
        this.tbvServiceRegistry.addSelectionChangedListener(new ISelectionChangedListener(){

			public void selectionChanged(SelectionChangedEvent event) {
				TableItem[] selections = theDialog.tableServiceRegistry.getSelection();
				if (selections.length > 0){
					theDialog.selectedServiceRegistry = (ServiceRegistryConfiguration)selections[0].getData();
					setMessage("Selected Service Registry " + theDialog.selectedServiceRegistry.getName());
					
				}
			}
        	
        });
        
        this.tbvServiceRegistry.addDoubleClickListener(new IDoubleClickListener(){
        	
			public void doubleClick(DoubleClickEvent event) {
				TableItem[] selection = tableServiceRegistry.getSelection();
				if ((selection != null) && (selection.length > 0)){
	    			selectedServiceRegistry = (ServiceRegistryConfiguration)((TableItem)selection[0]).getData();
	    			setMessage("Selected Service Registry " + theDialog.selectedServiceRegistry.getName());
	    			ServiceRegistryDialog aNewServiceRegistryDialog = new ServiceRegistryDialog(getShell(), selectedServiceRegistry);
	    			aNewServiceRegistryDialog.open();
	    			theDialog.tbvServiceRegistry.setInput(getServiceRegistryList());
	    			theDialog.tbvServiceRegistry.refresh(true);
	    			theDialog.internalSash.layout();
	    		}else{
	    			MessageDialog.openConfirm(
							getShell(),
							"Spagic",
							"Select a Service Registry");
	    		}
				
			}
	
        });
        return parent; 
	
	}
	
	protected void okPressed() {
		super.okPressed();
		
	}
	
	@Override
	protected void buttonPressed(int buttonId) {
		super.buttonPressed(buttonId);
	}


	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL,
				true);
	}
	
	public List<ServiceRegistryConfiguration> getServiceRegistryList(){
		List<ServiceRegistryConfiguration> serviceRegistryList = new ArrayList<ServiceRegistryConfiguration>();
		try{
			SAXReader reader = new SAXReader(false);
			Document serviceRegistryDocument = reader.read(SpagicUIActivator.getServiceRegistryFile());
			List<org.dom4j.Node> nodeList = serviceRegistryDocument.selectNodes("/SERVICE_REGISTRIES/SERVICE_REGISTRY");
			
			for (org.dom4j.Node n : nodeList){
				serviceRegistryList.add(new ServiceRegistryConfiguration(n.valueOf("@name"), 
																		n.valueOf("@inquiryUrl"),
																		n.valueOf("@publishUrl"),
																		n.valueOf("@user"),
																		n.valueOf("@password"),
																		n.valueOf("@defaultOrganitation")));
			}
		}catch (Exception e) {
			SpagicLogger.error("org.spagic.ui", e.getMessage(), e);
		}
		return serviceRegistryList;	
	}
}
