/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.TitleAreaDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

public class ProcessPropertiesDialog extends TitleAreaDialog {
	
	public class PropertyLabelProvider implements ITableLabelProvider
	{
		public PropertyLabelProvider() {
		}

		public String getColumnText(Object element, int column_index) {
			if (element instanceof Property) {
				// 0 - Property name
				// 1 - Value

				if (column_index == 0) {
					return ((Property) element).getKey();
				}
				if (column_index == 1) {
					return ((Property) element).getValue();
				}
			}
			return "";
		}

		public void addListener(ILabelProviderListener ilabelproviderlistener) {
		}

		public void dispose() {
		}

		public boolean isLabelProperty(Object obj, String s) {
			return false;
		}

		public void removeListener(ILabelProviderListener ilabelproviderlistener) {
		}

		public Image getColumnImage(Object element, int column_index) {
			return null;
		}
	}
	
	public class PropertyContentProvider implements
			IStructuredContentProvider {

		public PropertyContentProvider() {
			super();
		}

		public Object[] getElements(Object element) {

			List<Property> properties = (List<Property>)element;
			return properties.toArray();
		}

		public void dispose() {
		}

		public void inputChanged(Viewer viewer, Object old_object,
				Object new_object) {
		}
	}

	public static final int NEW_PROPERTY_ID = 99002;

    private SashForm internalSash  = null; 
    
    private Table tableProp = null;
    private TableViewer tbvProp = null;
    private List<Property> listProperties = null;
    private Properties parameters = null;
    private Property selectedProperty = null;
    
	public ProcessPropertiesDialog(Shell parentShell, Properties parameters) {
		super(parentShell);
		this.parameters = parameters;
	}

	/**
	   * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	   *      a title and a message
	   */
	  public void create() {
		super.create();
		setTitle("Process Properties");
	    setMessage("*** PLEASE ENSURE THAT THE DIAGRAM IS CLOSED BEFORE CHANGING THESE PROPERTIES ***");
	  }

	protected Control createDialogArea(Composite parent) {
		
        this.internalSash = new SashForm(parent,  SWT.VERTICAL | SWT.FLAT);
        this.internalSash.setLayoutData(new GridData(GridData.FILL_BOTH));
       	
        this.tableProp = new Table(internalSash, SWT.MULTI | SWT.FULL_SELECTION | SWT.BORDER);
        this.tableProp.setLinesVisible(true);
        this.tableProp.setHeaderVisible(true);
        TableColumn column = null;
		column = new TableColumn(tableProp, SWT.LEFT);
		column.setText("Property Name");
		column.setWidth(200);
		column = new TableColumn(tableProp, SWT.LEFT);
		column.setText("Value");
		column.setWidth(400);
		
        this.tbvProp = new TableViewer(tableProp);
        tbvProp.setContentProvider(new PropertyContentProvider());
        tbvProp.setLabelProvider(new PropertyLabelProvider());

        listProperties = getProperties(parameters);
        tbvProp.setInput(listProperties);
        
        final ProcessPropertiesDialog  theDialog = this;
        
        this.tableProp.addKeyListener(new KeyListener(){
        	public void keyPressed(org.eclipse.swt.events.KeyEvent e) {
        		System.out.println("KeyCode =" +  e.keyCode);
        		if (e.keyCode == 127){
        			TableItem[] tis = theDialog.tableProp.getSelection();
        			
        			if (tis.length > 0){
        				listProperties.remove((Property)tis[0].getData());
    					theDialog.tbvProp.refresh(true);
        			}
        		}
        	};
        	public void keyReleased(org.eclipse.swt.events.KeyEvent e) {
        		
        	};
        	
        });
        
        this.tbvProp.addSelectionChangedListener(new ISelectionChangedListener(){

			public void selectionChanged(SelectionChangedEvent event) {
				TableItem[] selections = theDialog.tableProp.getSelection();
				if (selections.length > 0){
					theDialog.selectedProperty = (Property)selections[0].getData();
					setMessage("Selected Property " + theDialog.selectedProperty.getKey());
					
				}
			}
        	
        });
        
        this.tbvProp.addDoubleClickListener(new IDoubleClickListener(){
        	
			public void doubleClick(DoubleClickEvent event) {
				TableItem[] selection = tableProp.getSelection();
				if ((selection != null) && (selection.length > 0)){
	    			selectedProperty = (Property)((TableItem)selection[0]).getData();
	    			setMessage("Selected Property " + theDialog.selectedProperty.getKey());
	    			PropertyDialog aNewXMLNamespaceDialog = new PropertyDialog(getShell(), selectedProperty);
	    			aNewXMLNamespaceDialog.open();
	    			theDialog.tbvProp.refresh(true);
	    			theDialog.internalSash.layout();
	    		}else{
	    			MessageDialog.openConfirm(
							getShell(),
							"Spagic",
							"Select a Property ");
	    		}
				
			}
	
        });
        return parent; 
	
	}
		
	@Override
	protected void buttonPressed(int buttonId) {
		if (buttonId == NEW_PROPERTY_ID){
			Property newProperty = new Property();
			PropertyDialog aNewXMLNamespaceDialog = new PropertyDialog(getShell(), newProperty);
			if (aNewXMLNamespaceDialog.open() == PropertyDialog.OK) {
				listProperties.add(newProperty);
			}
			
			this.tbvProp.refresh(true);
			this.internalSash.layout();
			
		} else if (buttonId == IDialogConstants.OK_ID) {
			parameters.clear();
			for (Property property : listProperties) {
				parameters.setProperty(property.getKey(), property.getValue());
			}
		}
		super.buttonPressed(buttonId);
	}


	protected void createButtonsForButtonBar(Composite parent) {
		
		createButton(parent, NEW_PROPERTY_ID, "New Property",
				false);
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL,
				true);
				
	}
	
	public List<Property> getProperties(Properties props) {
		List<Property> result = new ArrayList<Property>(props.size());
		for (Map.Entry<Object,Object> entry : props.entrySet()) {
			result.add(new Property((String)entry.getKey(), (String)entry.getValue()));
		}
		return result;
	}
	
}
