package org.spagic.utils;

import java.util.List;

import org.eclipse.emf.common.util.EMap;
import org.eclipse.stp.im.BasicProperty;
import org.eclipse.stp.im.ImFactory;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.Transition;
import org.eclipse.stp.im.constants.ImConstants;

public abstract class AbstractImPreprocessor implements ImPreprocessor {

	protected ImFactory factory = null;
	
	public AbstractImPreprocessor(){
		factory = ImFactory.eINSTANCE;
	}
	
	public void createTechnologyProperty(Process process, String technology) {
		BasicProperty technologyProperty = factory.createBasicProperty();
		technologyProperty.setKey(ImConstants.IM_PROCESS_TECHNOLOGY);
		technologyProperty.setValue(technology);
		process.getProperties().put(technologyProperty.getKey(),
				technologyProperty);
	}

	public static boolean isService(Step aStep, String serviceName) {
		return aStep.getServiceModel().getServiceName().equalsIgnoreCase(serviceName);
	}

	public static boolean isIncomingTransitionFromExternalEventHandlerBindingComponent(
			Transition t) {
		Step sourceStep = t.getSource();
		return isService(sourceStep, "ExternalEventService");

	}

	

	public void copyAllProperties(Step sourceStep, Step targetStep) {
		EMap<String, Property> sourceProperties = sourceStep
			.getProperties();
	
		for (String keyProp : sourceProperties.keySet()) {
				
				String propertyValue = (String)sourceProperties.get(keyProp).getValue();
				setPropertyOnStep(targetStep, keyProp, propertyValue);
				
		}
	}

	

	

	public boolean isServiceBinding(ServiceBinding serviceBinding, String name) {
		return name.equalsIgnoreCase( serviceBinding.getName() );
	}

	public void removeTransitionFromProcess(Process process, Transition transition) {
		process.getTransitions().remove(transition);
	}

	public void removeFromStepOutcomingTransition(Step step, Transition transition) {
		step.getSourceTransitions().remove(transition);
	}

	public void removeFromStepOutcomingTransition(Step step, List<Transition> transitions) {
		step.getSourceTransitions().removeAll(transitions);
	}

	public void removeFromStepIncomingTransition(Step step, Transition transition) {
		step.getTargetTransitions().remove(transition);
	}

	public void addServiceBinding(Step step, ServiceBinding serviceBinding) {
		step.getStepbindings().add(serviceBinding);
	}

	public Step createStep(StpIntermediateModel im, String stepName, String serviceBindingName) {
		Step newStep = factory.createStep();
		newStep.setName(stepName);
		addServiceBinding(newStep, findServiceBindingWithName(im, serviceBindingName));
		return newStep;
	}

	public int stepNumber(Process process) {
		return process.getSteps().size();
	}

	public ServiceBinding findServiceBindingWithName(StpIntermediateModel im, String name) {
		for (ServiceBinding aServiceBinding : im.getServicebindings()) {
			if (aServiceBinding.getName().equalsIgnoreCase(name)){
				return aServiceBinding;
			}
		}
		return null;
	}

	public Transition getFirstIncomingTransition(Step step) {
		if ( step.getTargetTransitions().size() > 0 )
			return step.getTargetTransitions().get(0);
		else
			return null;
	}

	public Transition getFirstOutcomingTransition(Step step) {
		if ( step.getSourceTransitions().size() > 0 )
			return step.getSourceTransitions().get(0);
		else
			return null;
	}

	public void setPropertyOnStep(Step step,
			String propertyName, String propertyValue) {
				BasicProperty newProperty = factory.createBasicProperty();
				newProperty.setKey(propertyName);
				newProperty.setValue(propertyValue);
				step.getProperties().put(propertyName, newProperty);
			}

	public void createTransitionAndAddToProcess(
			Process process, Step source, Step target) {
				
				Transition newTransition = factory.createTransition();
				newTransition.setSource(source);
				newTransition.setTarget(target);
				process.getTransitions().add(newTransition);
			
			}

	
	
}
