package org.spagic.utils.wsdl;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import org.dom4j.Attribute;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.XPath;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.dom4j.xpath.DefaultXPath;

public class XSDFilesFlattener {

	public static void main(String[] args) {
		if (args[0] == null || "".equals(args[0])
				|| args[1] == null || "".equals(args[1])) {
			System.out.println("ERROR: Wrong parameters.");
			usage();
			return;
		}
		
		flattenFiles(args[0], args[1]);
	}

	public static void flattenFiles(String origDirString, String flatDirString) {
		File origDir = new File(origDirString);
		if (!origDir.exists() || !origDir.isDirectory()) {
			System.out.println("ERROR: Wrong parameters: <source_dir> must exist.");
			return;
		}
		File flatDir = new File(flatDirString);

		flattenFiles(origDir, flatDir);
	}

	public static void flattenFiles(File origDir, File flatDir) {
		File xsdDir = new File(flatDir.getAbsolutePath() + File.separator + "xsd");

		if (!flatDir.exists()) {
			flatDir.mkdir();
			xsdDir.mkdir();
		} else {
			System.out.println("WARNING: <target_dir> already exists.");
			if (!xsdDir.exists()) {
				xsdDir.mkdir();
			}
		}

		visitAllFiles(origDir, flatDir, xsdDir);
	}
	
	public static void usage() {
		System.out.println("Usage: ");
		System.out.println("java XSDFilesFlattener <source_dir> <target_dir>");
		System.out.println("\t<target_dir> may not exists.");
	}

	public static void visitAllFiles(File dir, File flatDir, File xsdDir) {
		if (dir.isDirectory()) {
			String[] children = dir.list();
			for (int i = 0; i < children.length; i++) {
				visitAllFiles(new File(dir, children[i]), flatDir, xsdDir);
			}
		} else {
			process(dir, flatDir, xsdDir);
		}
	}

	private static void process(File dir, File flatDir, File xsdDir) {
		if (dir.getName().endsWith(".xsd")) {
			processXSD(dir, xsdDir);
		} else if (dir.getName().endsWith(".wsdl")) {
			processWSDL(dir, flatDir);
		} else {
			// skip
		}

	}

	private static void processWSDL(File dir, File destDir) {
		try {
			SAXReader reader = new SAXReader();
			Document doc = reader.read(dir);

			String includeXPath = "//xs:include/@schemaLocation";

			XPath includeXPathExpression = new DefaultXPath(includeXPath);

			HashMap<String, String> namespaceURI = new HashMap<String, String>();
			namespaceURI.put("xs", "http://www.w3.org/2001/XMLSchema");
			includeXPathExpression.setNamespaceURIs(namespaceURI);

			List nodeList = includeXPathExpression.selectNodes(doc);

			// Process the elements in the nodelist
			for (Object attributeObj : nodeList) {
				Attribute attribute = (Attribute) attributeObj;
				String schemaLocation = attribute.getValue();
				schemaLocation = schemaLocation.replace('\\', '/');
				int lastSlashIndex = schemaLocation.lastIndexOf('/');
				if (lastSlashIndex != -1
						&& lastSlashIndex + 1 < schemaLocation.length()) {
					attribute.setValue("./xsd/" //must use "/" here
							+ schemaLocation.substring(schemaLocation
									.lastIndexOf('/') + 1));
				}
			}

			writeToFile(doc, dir.getName(), destDir);

		} catch (DocumentException e) {
			e.printStackTrace();
		}
	}

	private static void processXSD(File dir, File destDir) {
		try {
			SAXReader reader = new SAXReader();
			Document doc = reader.read(dir);

			String includeXPath = "//xs:include/@schemaLocation";

			XPath includeXPathExpression = new DefaultXPath(includeXPath);

			HashMap<String, String> namespaceURI = new HashMap<String, String>();
			namespaceURI.put("xs", "http://www.w3.org/2001/XMLSchema");
			includeXPathExpression.setNamespaceURIs(namespaceURI);

			List nodeList = includeXPathExpression.selectNodes(doc);

			// Process the elements in the nodelist
			for (Object attributeObj : nodeList) {
				Attribute attribute = (Attribute) attributeObj;
				String schemaLocation = attribute.getValue();
				schemaLocation = schemaLocation.replace('\\', '/');
				int lastSlashIndex = schemaLocation.lastIndexOf('/');
				if (lastSlashIndex != -1
						&& lastSlashIndex + 1 < schemaLocation.length()) {
					attribute.setValue(schemaLocation.substring(schemaLocation
							.lastIndexOf('/') + 1));
				}
			}

			writeToFile(doc, dir.getName(), destDir);

		} catch (DocumentException e) {
			e.printStackTrace();
		}
	}

	public static void writeToFile(Document doc, String name, File destDir) {
		File xsdFile = new File(destDir.getAbsolutePath() + File.separator
				+ name);
		writeXmlFile(doc, xsdFile);
	}

	public static void writeXmlFile(Document doc, File destFile) {
		try {
			XMLWriter writer = new XMLWriter(new FileWriter(destFile));
			writer.write(doc);
			writer.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
